#pragma once

#include <string>
#include <functional>
#include <cinttypes>

#include "driver/ledc.h"

#include "Metrics.hpp"
#include "PWMChannel.hpp"



namespace SiliconTorch {

  extern const char *TAG;

  typedef std::function<void()> PacketHandledCallback;
  typedef std::function<float(float)> GammaCorrector;

  class FxCyanF {
    public:
      FxCyanF(uint32_t baseChannel = 0);

      bool addChannel(uint8_t gpio);

      void setPWM(uint8_t channel, float value);

      void setGammaCorrector(GammaCorrector gammaCorrector);

      uint8_t getChannelCount();

      void setBaseChannel(uint16_t baseChannel);
      uint16_t getBaseChannel();

      bool setFrequency(uint32_t frq_hz);
      uint32_t getFrequency();

      bool setResolution(uint8_t res_bits);
      uint8_t getResolution();

      bool setFrqRes(uint32_t frq_hz, uint8_t res_bits);

      void setPacketHandledCallback(PacketHandledCallback callback);

      bool handleUnicast(const char *data, std::size_t length);
      bool handleBroadcast(const char *data, std::size_t length);

      std::string genDeviceTopic(const char *suffix);
      std::string genServiceTopic(const char *suffix);

      void publishFrqRes();
      void publishFrequency();
      void publishResolution();
      void publishBaseChannel();
      void publishChannelCount();
      void publishListenerInfo();

    private:

      uint32_t frequency = 1000;  // Hz
      uint8_t resolution =   10;  // bits

      uint32_t baseChannel;
      uint8_t channelsConfigured = 0;

      Impl::PWMChannel **channels;

      ledc_timer_config_t timer_cfg;

      GammaCorrector gammaCorrector = [](float f) { return f*f; };

      PacketHandledCallback packetCallback = []() {};
      void callPacketCallback();

      Metrics::Metrics metrics;
      Metrics::Incrementer frameCntInc;
      Metrics::Incrementer errorCntInc;
  };

}
